<?php

/**
 * CApplicationSetting Component
 *
 * This component is the one that manages applications and serves models.
 *
 * Example configuration:
 *
 * protected/config/main.php
 *
 * components=>array(
 * ...
 *   'settings'=>array(
 *     'class'=>'CApplicationSettings',
 *     'settingsTable'=>'{{setting}}',
 *     'settingsIdColumn'=>'name',
 *     'applications'=>array(
 *       'main'=>array(
 *         'settingsTable'=>'{{module_setting}}',
 *         'settingsIdColumn'=>'name',
 *         'settingsCategoryColumn'=>'module',
 *        ),
 *       'forum'=>array(
 *         'isCategoryOf'=>'main' //Inherits all the config from 'main' application.
 *       ),
 *       'blog'=>array(
 *         'isCategoryOf'=>'main' //Inherits all the config from 'main' application.
 *       ),
 *      'anotherForum'=>array(
 *         'settingsTable'=>'forum_settings',
 *         'settingsIdColumn'=>'setting_name',
 *         'settingsValueColumn'=>'setting_value',
 *         'connection'=>'forum_db_conn',
 *       ),
 *     ),
 *   ),
 *...
 * ),
 *
 * This way you can access forum_home_url these ways:
 *
 * Yii::app()->settings->forum->forum_home_url  // Condition will be 'name = `forum_home_url` AND module = `forum`'
 * Yii::app()->settings->main->forum_home_url  // Condition will be 'name = `forum_home_url`'
 * Yii::app()->settings->blog->forum_home_url  // Will throw Exception because doesn't exists in condition 'module = `blog`'
 *
 * And anotherForum is another application.
 *
 * Yii::app()->settings->anotherForum->someSetting
 * Yii::app()->settings->anotherForum->someSetting = value
 *
 *
 * Report any bug at http://tracker.fusiondev.com.ar
 *
 * @author Jorge Sivil <jsivil@fusiondev.com.ar>
 * @since CApplicationSettings-1.0
 */
class CApplicationSettings extends CApplicationComponent
{
  public $cache = false;
  public $connection = 'db';
  public $settingsTable = 'settings';
  public $settingsIdColumn = 'id';
  public $settingsValueColumn = 'value';
  public $applications = array();
  public $settingsCategoryColumn = null;
  /**
   * @var array Array models are saved to avoid multiple instantiation of the same model.
   */
  private static $_models = array();

  /**
   * Magic setter that acts as application's name.
   *
   * If it exists, returns a intermediary model between
   * this Component and the Active Record class.
   *
   * @param string $app
   * @return null|ApplicationSetting
   */
  public function __get( $app )
  {
    if( $this->applications && array_key_exists($app, $this->applications) )
    {
      self::$_models[$app] = new ApplicationSetting($this->makeConfig($app));
      return self::$_models[$app];
    }
    else
    {
      return parent::__get( $app );
    }
  }

  /**
   * Makes config array.
   *
   * Returns a config array that will be used for CApplicationSetting model instantiation.
   *
   * @param $app
   * @return array
   */
  private function makeConfig($app)
  {
    $settingsCategoryColumn = isset( $this->applications[$app]['settingsCategoryColumn'] )
      ? $this->applications[$app]['settingsCategoryColumn']
      : $this->settingsCategoryColumn;

    if(isset($this->applications[$app]['isCategoryOf']))
    {
      $category = $app;
      $app = $this->applications[$app]['isCategoryOf'];
    }
    else
      $settingsCategoryColumn = '';
    // To allow parent app still access all the values, other way a condition comparing category will be raised.

    $connection = isset( $this->applications[$app]['connection'] )
      ? $this->applications[$app]['connection']
      : $this->connection;

    $settingsTable = isset( $this->applications[$app]['settingsTable'] )
      ? $this->applications[$app]['settingsTable']
      : $this->settingsTable;

    $settingsIdColumn = isset( $this->applications[$app]['settingsIdColumn'] )
      ? $this->applications[$app]['settingsIdColumn']
      : $this->settingsIdColumn;

    $settingsValueColumn = isset( $this->applications[$app]['settingsValueColumn'] )
      ? $this->applications[$app]['settingsValueColumn']
      : $this->settingsValueColumn;


    return $config = array(
      'connection'=>$connection,
      'settingsTable'=>$settingsTable,
      'settingsIdColumn'=>$settingsIdColumn,
      'settingsValueColumn'=>$settingsValueColumn,
      'settingsCategoryColumn'=>$settingsCategoryColumn,
      'categoryName'=>isset($category) ? $category : null,
    );
  }
}

/**
 * CApplicationSetting Intermediary model
 *
 * Class that will act as nexus between CApplicationSettings Component and Active Record model
 *
 * @author Jorge Sivil <jsivil@fusiondev.com.ar>
 * @since CApplicationSettings-1.0
 */
class ApplicationSetting
{
  public static $settingsIdColumn;
  public static $settingsValueColumn;
  public static $settingsCategoryColumn;
  public static $categoryName;

  public function __construct( $config = array() )
  {
    if( $config )
    {
      self::$settingsIdColumn = $config['settingsIdColumn'];
      self::$settingsValueColumn = $config['settingsValueColumn'];
      self::$settingsCategoryColumn = $config['settingsCategoryColumn'];
      self::$categoryName = $config['categoryName'];
      new ApplicationSettingAR('insert',$config); // Initialize ApplicationSettingAR properties (table, connection, etc)
    }
  }

  /**
   * Magic getter
   *
   * @param $attr string The attribute that wants to get.
   * @return string|null
   */
  public function __get($attr)
  {
    $model = $this->getModel($attr);
    if( $model !== null )
    {
      return $model->{self::$settingsValueColumn};
    }
    if(self::$categoryName)
      throw new CHttpException('Property \'' . $attr . '\' in category \'' . self::$categoryName . '\' does not exists.', 503 );
    else
      throw new CHttpException('Property \'' . $attr . '\' does not exists.', 503 );
  }

  /**
   * Magic setter
   *
   * @param $attr string The attribute that wants to set.
   * @param $value string The value that the attribute should get saved on.
   * @throws CHttpException If value doesn't get saved correctly or it doesn't exists.
   */
  public function __set($attr, $value)
  {
    $model = $this->getModel($attr);
    if( $model === null ) {
      throw new CHttpException('Could not get property \'' . $attr . '\' to assign value \'' . $value . '\'.', 503 );
    }
    $model->{self::$settingsValueColumn} = $value;
    if( !$model->save() ) {
      throw new CHttpExcepton('Could not save value \'' . $value . '\' to property \'' . $attr . '\'.', 503 );
    }
  }

  /**
   * Gets CApplicationSettings' ActiveRecord model.
   *
   * @param $attr
   * @return CActiveRecord|null
   */
  private function getModel($attr)
  {
    if( self::$settingsCategoryColumn )
    {
      $model = ApplicationSettingAR::model()->find(
        self::$settingsIdColumn . ' = :propName AND ' . self::$settingsCategoryColumn . ' = :catName',
        array(':propName'=>$attr,':catName'=>self::$categoryName)
      );
    }
    else
    {
      $model = ApplicationSettingAR::model()->find(self::$settingsIdColumn . ' = :propName',array(':propName'=>$attr));
    }
    return $model;
  }
}

/**
 * CApplicationSetting Active Record
 *
 * Class that will act as nexus between CApplicationSettings and the DB.
 *
 * @author Jorge Sivil <jsivil@fusiondev.com.ar>
 * @since CApplicationSettings-1.0
 */
class ApplicationSettingAR extends CActiveRecord
{
  public static $settingsTable;
  public static $settingsIdColumn;
  public static $settingsValueColumn;

  public static function model($className=__CLASS__)
  {
    return parent::model($className);
  }

  public function tableName()
  {
    return self::$settingsTable;
  }

  public function __construct( $scenario='insert', $config = array() )
  {
    if( $config )
    {
      self::$db = Yii::app()->$config['connection'];
      self::$settingsTable = $config['settingsTable'];
      self::$settingsIdColumn = $config['settingsIdColumn'];
      self::$settingsValueColumn = $config['settingsValueColumn'];
    }
    parent::__construct($scenario);
  }
}
